@@ -314,12 +314,13 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
     @starts.setter
     def starts(self, value):
         value = self._util_toarray(value, self.INDEXTYPE, self.numpy.ndarray)
-        if not self._util_isintegertype(value.dtype.type):
-            raise TypeError("starts must have integer dtype")
-        if len(value.shape) == 0:
-            raise ValueError("starts must have at least one dimension")
-        if (value < 0).any():
-            raise ValueError("starts must be a non-negative array")
+        if self.check_prop_valid:
+            if not self._util_isintegertype(value.dtype.type):
+                raise TypeError("starts must have integer dtype")
+            if len(value.shape) == 0:
+                raise ValueError("starts must have at least one dimension")
+            if (value < 0).any():
+                raise ValueError("starts must be a non-negative array")
         self._starts = value
         self._offsets, self._counts, self._parents = None, None, None
         self._isvalid = False
@@ -331,12 +332,13 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
     @stops.setter
     def stops(self, value):
         value = self._util_toarray(value, self.INDEXTYPE, self.numpy.ndarray)
-        if not self._util_isintegertype(value.dtype.type):
-            raise TypeError("stops must have integer dtype")
-        if len(value.shape) == 0:
-            raise ValueError("stops must have at least one dimension")
-        if (value < 0).any():
-            raise ValueError("stops must be a non-negative array")
+        if self.check_prop_valid:
+            if not self._util_isintegertype(value.dtype.type):
+                raise TypeError("stops must have integer dtype")
+            if len(value.shape) == 0:
+                raise ValueError("stops must have at least one dimension")
+            if (value < 0).any():
+                raise ValueError("stops must be a non-negative array")
         self._stops = value
         self._offsets, self._counts, self._parents = None, None, None
         self._isvalid = False
@@ -368,12 +370,13 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
     @offsets.setter
     def offsets(self, value):
         value = self._util_toarray(value, self.INDEXTYPE, self.numpy.ndarray)
-        if not self._util_isintegertype(value.dtype.type):
-            raise TypeError("offsets must have integer dtype")
-        if len(value.shape) != 1 or (value < 0).any():
-            raise ValueError("offsets must be a one-dimensional, non-negative array")
-        if len(value) == 0:
-            raise ValueError("offsets must be non-empty")
+        if self.check_prop_valid:
+            if not self._util_isintegertype(value.dtype.type):
+                raise TypeError("offsets must have integer dtype")
+            if len(value.shape) != 1 or (value < 0).any():
+                raise ValueError("offsets must be a one-dimensional, non-negative array")
+            if len(value) == 0:
+                raise ValueError("offsets must be non-empty")
         self._starts = value[:-1]
         self._stops = value[1:]
         self._offsets = value
@@ -390,12 +393,13 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
     @counts.setter
     def counts(self, value):
         value = self._util_toarray(value, self.INDEXTYPE, self.numpy.ndarray)
-        if not self._util_isintegertype(value.dtype.type):
-            raise TypeError("counts must have integer dtype")
-        if len(value.shape) == 0:
-            raise ValueError("counts must have at least one dimension")
-        if (value < 0).any():
-            raise ValueError("counts must be a non-negative array")
+        if self.check_prop_valid:
+            if not self._util_isintegertype(value.dtype.type):
+                raise TypeError("counts must have integer dtype")
+            if len(value.shape) == 0:
+                raise ValueError("counts must have at least one dimension")
+            if (value < 0).any():
+                raise ValueError("counts must be a non-negative array")
         offsets = self.counts2offsets(value.reshape(-1))
         self._starts = offsets[:-1].reshape(value.shape)
         self._stops = offsets[1:].reshape(value.shape)
@@ -417,10 +421,11 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
     @parents.setter
     def parents(self, value):
         value = self._util_toarray(value, self.INDEXTYPE, self.numpy.ndarray)
-        if not self._util_isintegertype(value.dtype.type):
-            raise TypeError("parents must have integer dtype")
-        if len(value.shape) == 0:
-            raise ValueError("parents must have at least one dimension")
+        if self.check_prop_valid:
+            if not self._util_isintegertype(value.dtype.type):
+                raise TypeError("parents must have integer dtype")
+            if len(value.shape) == 0:
+                raise ValueError("parents must have at least one dimension")
         self._starts, self._stops = self.parents2startsstops(value)
         self._offsets, self._counts = None, None
         self._parents = value
@@ -440,31 +445,36 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
         if not self._isvalid:
             if self.offsetsaliased(self._starts, self._stops):
                 self._offsets = self._starts.base
-                if not (self._offsets[1:] >= self._offsets[:-1]).all():
-                    raise ValueError("offsets must be monatonically increasing")
-                if self._offsets.max() > len(self._content):
-                    raise ValueError("maximum offset {0} is beyond the length of the content ({1})".format(self._offsets.max(), len(self._content)))
+                if self.check_whole_valid:
+                    if not (self._offsets[1:] >= self._offsets[:-1]).all():
+                        raise ValueError("offsets must be monatonically increasing")
+                    if self._offsets.max() > len(self._content):
+                        raise ValueError("maximum offset {0} is beyond the length of the content ({1})".format(self._offsets.max(), len(self._content)))
 
             else:
-                self._validstartsstops(self._starts, self._stops)
+                if self.check_whole_valid:
+                    self._validstartsstops(self._starts, self._stops)
                 nonempty = (self._starts != self._stops)
                 starts = self._starts[nonempty].reshape(-1)
-                if len(starts) != 0 and starts.reshape(-1).max() >= len(self._content):
-                    raise ValueError("maximum start ({0}) is at or beyond the length of the content ({1})".format(starts.reshape(-1).max(), len(self._content)))
+                if self.check_whole_valid:
+                    if len(starts) != 0 and starts.reshape(-1).max() >= len(self._content):
+                        raise ValueError("maximum start ({0}) is at or beyond the length of the content ({1})".format(starts.reshape(-1).max(), len(self._content)))
                 stops = self._stops[nonempty].reshape(-1)
-                if len(stops) != 0 and stops.reshape(-1).max() > len(self._content):
-                    raise ValueError("maximum stop ({0}) is beyond the length of the content ({1})".format(self._stops.reshape(-1).max(), len(self._content)))
+                if self.check_whole_valid:
+                    if len(stops) != 0 and stops.reshape(-1).max() > len(self._content):
+                        raise ValueError("maximum stop ({0}) is beyond the length of the content ({1})".format(self._stops.reshape(-1).max(), len(self._content)))
 
             self._isvalid = True
 
-    @staticmethod
-    def _validstartsstops(starts, stops):
-        if len(starts) > len(stops):
-            raise ValueError("starts must have the same (or shorter) length than stops")
-        if starts.shape[1:] != stops.shape[1:]:
-            raise ValueError("starts and stops must have the same dimensionality (shape[1:])")
-        if not (stops >= starts).all():
-            raise ValueError("stops must be greater than or equal to starts")
+    @classmethod
+    def _validstartsstops(cls, starts, stops):
+        if cls.check_whole_valid:
+            if len(starts) > len(stops):
+                raise ValueError("starts must have the same (or shorter) length than stops")
+            if starts.shape[1:] != stops.shape[1:]:
+                raise ValueError("starts and stops must have the same dimensionality (shape[1:])")
+            if not (stops >= starts).all():
+                raise ValueError("stops must be greater than or equal to starts")
 
     def __iter__(self, checkiter=True):
         if checkiter:
@@ -1319,7 +1329,7 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
         return self.copy(starts=newstarts, stops=newstops, content=flatout)
 
     @awkward.util.bothmethod
-    def concatenate(isclassmethod, cls_or_self, arrays):
+    def concatenate(isclassmethod, cls_or_self, arrays, axis=0):
         if isclassmethod: 
             cls = cls_or_self
             if not all(isinstance(x, JaggedArray) for x in arrays):
@@ -1334,6 +1344,12 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
         for x in arrays:
             x._valid()
 
+        if axis == 1:
+            return cls._concatenate_axis1(arrays)
+
+        if axis > 1:
+            raise NotImplementedError
+
         starts = cls.numpy.concatenate([x._starts for x in arrays])
         stops = cls.numpy.concatenate([x._stops for x in arrays])
         content = cls._util_concatenate([x._content for x in arrays])
@@ -1351,8 +1367,50 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
         return cls(starts, stops, content)
 
     @classmethod
-    def stack(cls, first, *rest):    # each item in first followed by second, etc.
-        raise NotImplementedError
+    def _concatenate_axis1(cls, arrays):
+
+        if len(arrays) == 0:
+            raise ValueError("at least one array must be provided")   # this can only happen in the classmethod case
+        if any(len(a) != len(arrays[0]) for a in arrays):
+            raise ValueError("cannot concatenate JaggedArrays of different lengths with axis=1")
+        if any(len(a.starts.shape) > 1 for a in arrays):
+            raise NotImplementedError
+
+        np = cls.numpy
+
+        flatarrays = [a.flatten() for a in arrays]
+        n = len(arrays)
+
+        if any(not isinstance(a, np.ndarray) for a in flatarrays):
+            raise NotImplementedError
+
+        # the first step is to get the starts and stops for the stacked structure
+        counts = np.vstack([a.counts for a in arrays])
+        flat_counts = counts.T.flatten()
+        offsets = cls.counts2offsets(flat_counts)
+        starts, stops = offsets[:-1], offsets[1:]
+
+        # find most general type with a tentative sum which implements the right type-promotion,
+        # except for booleans which would get promoted to integers when summing
+        dtype = np.dtype(sum([x[0] for x in flatarrays if len(x) != 0]), False)
+        allbools = not np.any([a.dtype != np.dtype(bool) for a in flatarrays])
+        dtype = np.dtype(bool) if allbools else dtype
+
+        n_content = sum([len(a) for a in flatarrays])
+
+        # use masks for each of the arrays so we can fill the stacked content array at the right indices
+        content = np.zeros(n_content, dtype=dtype)
+        for i in range(n):
+            working_array = np.zeros(n_content+1, dtype=cls.INDEXTYPE)
+            starts_i = starts[i::n]
+            stops_i = stops[i::n]
+            not_empty = starts_i != stops_i
+            working_array[starts_i[not_empty]] += 1
+            working_array[stops_i[not_empty]] -= 1
+            mask = np.array(np.cumsum(working_array)[:-1], dtype=cls.MASKTYPE)
+            content[mask] = flatarrays[i]
+
+        return cls(starts[::n], stops[n-1::n], content)
 
     @awkward.util.bothmethod
     def zip(isclassmethod, cls_or_self, columns1={}, *columns2, **columns3):
